﻿using System;
using System.Windows;
using System.Windows.Ink;

namespace TestProject
{
    /// <summary>
    /// 지우개 도구 윈도우
    /// </summary>
    public class EraserToolWindow : StylusToolWindow
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 지우개 모양 - EraserShape

        /// <summary>
        /// 지우개 모양
        /// </summary>
        public StylusShape EraserShape
        {
            set
            {
                this.heightTextBox.Text = (0.75 * value.Height).ToString("F1");
                this.widthTextBox.Text  = (0.75 * value.Width ).ToString("F1");
                this.angleTextBox.Text  = value.Rotation.ToString();

                if(value is EllipseStylusShape)
                {
                    this.ellipseRadioButton.IsChecked = true;
                }
                else
                {
                    this.rectangleRadioButton.IsChecked = true;
                }
            }
            get
            {
                StylusShape stylusShape;

                double width  = Double.Parse(this.widthTextBox.Text ) / 0.75;
                double height = Double.Parse(this.heightTextBox.Text) / 0.75;
                double angle  = Double.Parse(this.angleTextBox.Text );

                if((bool)this.ellipseRadioButton.IsChecked)
                {
                    stylusShape = new EllipseStylusShape(width, height, angle);
                }
                else
                {
                    stylusShape = new RectangleStylusShape(width, height, angle);
                }

                return stylusShape;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - EraserToolWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public EraserToolWindow()
        {
            Title = "지우개 도구";

            this.ignorePressureCheckBox.Visibility = Visibility.Collapsed;
            this.highlighterCheckBox.Visibility    = Visibility.Collapsed;
            this.colorListBox.Visibility           = Visibility.Collapsed;
        }

        #endregion
    }
}