﻿using Microsoft.Win32;
using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public
 
        #region Field

        /// <summary>
        /// 캔버스 너비 (8인치)
        /// </summary>
        public static readonly double CanvasWidth = 8 * 96;

        /// <summary>
        /// 캔버스 높이 (8인치)
        /// </summary>
        public static readonly double CanvasHeight = 8 * 96;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            double y = 96;

            while(y < CanvasHeight)
            {
                Line line = new Line();

                line.X1     = 0;
                line.Y1     = y;
                line.X2     = CanvasWidth;
                line.Y2     = y;
                line.Stroke = Brushes.LightBlue;

                this.inkCanvas.Children.Add(line);

                y += 24; // 0.25인치
            }

            if(Tablet.TabletDevices.Count == 0)
            {
                this.eraserModeMenuItem.Visibility = Visibility.Collapsed;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region New 실행시 처리하기 - New_Executed(sender, e)

        /// <summary>
        /// New 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void New_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            this.inkCanvas.Strokes.Clear();
        }

        #endregion
        #region Open 실행시 처리하기 - Open_Executed(sender, e)

        /// <summary>
        /// Open 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Open_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();

            openFileDialog.CheckFileExists = true;
            openFileDialog.Filter          = "Ink Serialized Format (*.isf)|*.isf|All files (*.*)|*.*";

            if((bool)openFileDialog.ShowDialog(this))
            {
                try
                {
                    FileStream fileStream = new FileStream(openFileDialog.FileName, FileMode.Open, FileAccess.Read);

                    this.inkCanvas.Strokes = new StrokeCollection(fileStream);

                    fileStream.Close();
                }
                catch(Exception exception)
                {
                    MessageBox.Show(exception.Message, Title);
                }
            }
        }

        #endregion
        #region Save 실행시 처리하기 - Save_Executed(sender, e)

        /// <summary>
        /// Save 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Save_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();

            saveFileDialog.Filter = "Ink Serialized Format (*.isf)|*.isf|XAML Drawing File (*.xaml)|*.xaml|All files (*.*)|*.*";

            if((bool)saveFileDialog.ShowDialog(this))
            {
                try
                {
                    FileStream fileStream = new FileStream(saveFileDialog.FileName, FileMode.Create, FileAccess.Write);

                    if(saveFileDialog.FilterIndex == 1 || saveFileDialog.FilterIndex == 3)
                    {
                        this.inkCanvas.Strokes.Save(fileStream);
                    }
                    else
                    {
                        DrawingGroup drawingGroup = new DrawingGroup();

                        foreach(Stroke stroke in this.inkCanvas.Strokes)
                        {
                            Color color = stroke.DrawingAttributes.Color;

                            if(stroke.DrawingAttributes.IsHighlighter)
                            {
                                color = Color.FromArgb(128, color.R, color.G, color.B);
                            }

                            drawingGroup.Children.Add
                            (
                                new GeometryDrawing
                                (
                                    new SolidColorBrush(color),
                                    null,
                                    stroke.GetGeometry()
                                )
                            );
                        }

                        XamlWriter.Save(drawingGroup, fileStream);
                    }

                    fileStream.Close();
                }
                catch(Exception exception)
                {
                    MessageBox.Show(exception.Message, Title);
                }
            }
        }

        #endregion
        #region Close 실행시 처리하기 - Close_Executed(sender, e)

        /// <summary>
        /// Close 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Close_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            Close();
        }

        #endregion

        #region Edit 메뉴 항목 하위 메뉴 오픈시 처리하기 - editMenuItem_SubmenuOpened(sender, e)

        /// <summary>
        /// Edit 메뉴 항목 하위 메뉴 오픈시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void editMenuItem_SubmenuOpened(object sender, RoutedEventArgs e)
        {
            this.formatMenuItem.IsEnabled = this.inkCanvas.GetSelectedStrokes().Count > 0;
        }

        #endregion
        #region Cut 실행시 처리하기 - Cut_Executed(sender, e)
        
        /// <summary>
        /// Cut 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Cut_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            this.inkCanvas.CutSelection();
        }

        #endregion
        #region Cut 실행 가능 여부 처리하기 - Cut_CanExecute(sender, e)

        /// <summary>
        /// Cut 실행 가능 여부 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Cut_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = this.inkCanvas.GetSelectedStrokes().Count > 0;
        }

        #endregion
        #region Copy 실행시 처리하기 - Copy_Executed(sender, e)

        /// <summary>
        /// Copy 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Copy_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            this.inkCanvas.CopySelection();
        }

        #endregion
        #region Paste 실행시 처리하기 - Paste_Executed(sender, e)

        /// <summary>
        /// Paste 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Paste_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            this.inkCanvas.Paste();
        }

        #endregion
        #region Paste 실행 가능 여부 처리하기 - Paste_CanExecute(sender, e)

        /// <summary>
        /// Paste 실행 가능 여부 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Paste_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = this.inkCanvas.CanPaste();
        }

        #endregion
        #region Delete 실행시 처리하기 - Delete_Executed(sender, e)

        /// <summary>
        /// Delete 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Delete_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            foreach(Stroke stroke in inkCanvas.GetSelectedStrokes())
            {
                this.inkCanvas.Strokes.Remove(stroke);
            }
        }

        #endregion
        #region Select All 실행시 처리하기 - SelectAll_Executed(sender, e)

        /// <summary>
        /// Select All 실행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void SelectAll_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            this.inkCanvas.Select(this.inkCanvas.Strokes);
        }

        #endregion
        #region Format 메뉴 항목 클릭시 처리하기 - formatMenuItem_Click(sender, e)

        /// <summary>
        /// Format 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void formatMenuItem_Click(object sender, RoutedEventArgs e)
        {
            StylusToolWindow stylusToolWindow = new StylusToolWindow();

            stylusToolWindow.Owner = this;
            stylusToolWindow.Title = "포맷 선택";

            StrokeCollection strokeCollection = this.inkCanvas.GetSelectedStrokes();

            if(strokeCollection.Count > 0)
            {
                stylusToolWindow.DrawingAttributes = strokeCollection[0].DrawingAttributes;
            }
            else
            {
                stylusToolWindow.DrawingAttributes = inkCanvas.DefaultDrawingAttributes;
            }

            if((bool)stylusToolWindow.ShowDialog().GetValueOrDefault())
            {
                foreach(Stroke stroke in strokeCollection)
                {
                    stroke.DrawingAttributes = stylusToolWindow.DrawingAttributes;
                }
            }
        }

        #endregion

        #region 스타일러스 모드 메뉴 항목 하위 메뉴 오픈시 처리하기 - stylusModeMenuItem_SubmenuOpened(sender, e)

        /// <summary>
        /// 스타일러스 모드 메뉴 항목 하위 메뉴 오픈시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void stylusModeMenuItem_SubmenuOpened(object sender, RoutedEventArgs e)
        {
            MenuItem menuItem = sender as MenuItem;

            foreach(MenuItem childMenuItem in menuItem.Items)
            {
                childMenuItem.IsChecked = this.inkCanvas.EditingMode == (InkCanvasEditingMode)childMenuItem.Tag;
            }
        }

        #endregion
        #region 스타일러스 모드 자식 메뉴 항목 클릭시 처리하기 - stylusModeChildMenuItem_Click(sender, e)

        /// <summary>
        /// 스타일러스 모드 자식 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void stylusModeChildMenuItem_Click(object sender, RoutedEventArgs e)
        {
            MenuItem menuItem = sender as MenuItem;

            this.inkCanvas.EditingMode = (InkCanvasEditingMode)menuItem.Tag;
        }

        #endregion
        #region 지우개 모드 메뉴 항목 하위 메뉴 오픈시 처리하기 - eraserModeMenuItem_SubmenuOpened(sender, e)

        /// <summary>
        /// 지우개 모드 메뉴 항목 하위 메뉴 오픈시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void eraserModeMenuItem_SubmenuOpened(object sender, RoutedEventArgs e)
        {
            MenuItem menuItem = sender as MenuItem;

            foreach(MenuItem childMenuItem in menuItem.Items)
            {
                childMenuItem.IsChecked = this.inkCanvas.EditingModeInverted == (InkCanvasEditingMode)childMenuItem.Tag;
            }
        }

        #endregion
        #region 지우개 모드 자식 메뉴 항목 클릭시 처리하기 - eraserModeChildMenuItem_Click(sender, e)

        /// <summary>
        /// 지우개 모드 자식 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void eraserModeChildMenuItem_Click(object sender, RoutedEventArgs e)
        {
            MenuItem menuItem = sender as MenuItem;

            this.inkCanvas.EditingModeInverted = (InkCanvasEditingMode)menuItem.Tag;
        }

        #endregion

        #region 스타일러스 도구 메뉴 항목 클릭시 처리하기 - stylusToolMenuItem_Click(sender, e)

        /// <summary>
        /// 스타일러스 도구 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void stylusToolMenuItem_Click(object sender, RoutedEventArgs e)
        {
            StylusToolWindow stylusToolWindow = new StylusToolWindow();

            stylusToolWindow.Owner             = this;
            stylusToolWindow.DrawingAttributes = this.inkCanvas.DefaultDrawingAttributes;

            if((bool)stylusToolWindow.ShowDialog().GetValueOrDefault())
            {
                this.inkCanvas.DefaultDrawingAttributes = stylusToolWindow.DrawingAttributes;
            }
        }

        #endregion
        #region 지우개 도구 메뉴 항목 클릭시 처리하기 - eraserToolMenuItem_Click(sender, e)

        /// <summary>
        /// 지우개 도구 메뉴 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void eraserToolMenuItem_Click(object sender, RoutedEventArgs e)
        {
            EraserToolWindow eraserToolWindow = new EraserToolWindow();

            eraserToolWindow.Owner       = this;
            eraserToolWindow.EraserShape = this.inkCanvas.EraserShape;

            if((bool)eraserToolWindow.ShowDialog().GetValueOrDefault())
            {
                this.inkCanvas.EraserShape = eraserToolWindow.EraserShape;
            }
        }

        #endregion
    }
}