﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Ink;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 스타일러스 도구 윈도우
    /// </summary>
    public partial class StylusToolWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 드로잉 어트리뷰트 - DrawingAttributes

        /// <summary>
        /// 드로잉 어트리뷰트
        /// </summary>
        public DrawingAttributes DrawingAttributes
        {
            set
            {
                this.heightTextBox.Text = (0.75 * value.Height).ToString("F1");
                this.widthTextBox.Text  = (0.75 * value.Width ).ToString("F1");
                this.angleTextBox.Text  = (180 * Math.Acos(value.StylusTipTransform.M11) / Math.PI).ToString("F1");

                this.ignorePressureCheckBox.IsChecked = value.IgnorePressure;
                this.highlighterCheckBox.IsChecked    = value.IsHighlighter;

                if(value.StylusTip == StylusTip.Ellipse)
                {
                    this.ellipseRadioButton.IsChecked = true;
                }
                else
                {
                    this.rectangleRadioButton.IsChecked = true;
                }

                this.colorListBox.SelectedColor = value.Color;

                this.colorListBox.ScrollIntoView(this.colorListBox.SelectedColor);
            }
            get
            {
                DrawingAttributes drawingAttributes = new DrawingAttributes();

                drawingAttributes.Height = Double.Parse(this.heightTextBox.Text) / 0.75;
                drawingAttributes.Width  = Double.Parse(this.widthTextBox.Text ) / 0.75;

                drawingAttributes.StylusTipTransform = new RotateTransform(Double.Parse(this.angleTextBox.Text)).Value;

                drawingAttributes.IgnorePressure = (bool)this.ignorePressureCheckBox.IsChecked;
                drawingAttributes.IsHighlighter  = (bool)this.highlighterCheckBox.IsChecked;
                drawingAttributes.StylusTip      = (bool)this.ellipseRadioButton.IsChecked ? StylusTip.Ellipse : StylusTip.Rectangle;

                drawingAttributes.Color = this.colorListBox.SelectedColor;

                return drawingAttributes;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - StylusToolWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public StylusToolWindow()
        {
            InitializeComponent();

            this.widthTextBox.TextChanged  += textBox_TextChanged;
            this.heightTextBox.TextChanged += textBox_TextChanged;
            this.angleTextBox.TextChanged  += textBox_TextChanged;

            this.widthTextBox.Focus();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 텍스트 박스 텍스트 변경시 처리하기 - textBox_TextChanged(sender, e)

        /// <summary>
        /// 텍스트 박스 텍스트 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void textBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            double width;
            double height;
            double angle;

            this.okButton.IsEnabled = Double.TryParse(this.widthTextBox.Text, out width)   &&
                                      width / 0.75 >= DrawingAttributes.MinWidth           &&
                                      width / 0.75 <= DrawingAttributes.MaxWidth           &&
                                      Double.TryParse(this.heightTextBox.Text, out height) &&
                                      height / 0.75 >= DrawingAttributes.MinHeight         &&
                                      height / 0.75 <= DrawingAttributes.MaxHeight         &&
                                      Double.TryParse(this.angleTextBox.Text, out angle);
        }

        #endregion
        #region OK 버튼 클릭시 처리하기 - okButton_Click(sender, e)

        /// <summary>
        /// OK 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void okButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }

        #endregion
    }
}